/*************************************************************************
 *
 *	AUTHOR:	Wilson Tham
 *
 *	TITLE:	checkbox.h
 *
 *	DATE:	April 25, 1993
 *
 *	PURPOSE: Declaration of checkbox class and constants.
 *
 *************************************************************************/


#ifndef _CHECKBOX_H
#define _CHECKBOX_H


#include <stdlib.h>
#include "typedefs.h"
#include "scrnobj.h"
#include "viewport.h"
#include "display.h"
#include "messages.h"

typedef bool BOOLEAN;
enum Type {Square, Circle};

				/*************************
				 *                       *
				 * Constant Declarations *
				 *                       *
				 *************************/

const int CBORDERX	= 4;  // Border space from box to edge
const int CBORDERY	= 4;  // " "
const int CFRAME	= 2;  // Additional highlighting of checkbox when Active
const int CFONT	= SMALL_FONT;
const int CFONT_SIZE= 5;
const int BOXSIZE	= 10; // For both circle and square... a box dimension
const int FILLSIZE	= 1;  // Border of fill graphics from box
const int BOXBORDER	= 4;  // Space between box and text.
const int BOXCOLOR	= LIGHTGRAY;
const int OFFCOLOR	= DARKGRAY;

class CheckBox:public ScrnObj{
	int string_height, string_width, COLORFILL, Type;
	char	string[80];
protected:
	int  X2, Y2, xdim, ydim;
	int	Color, BkColor, IdNum;
	BOOLEAN Activated, BStatus, Able;
	CheckBox (void) { }
public:
	CheckBox				(char*, int, int, int, int);
	CheckBox				(char*, int, int, int, int, int, int);
	// Default fillcolor set...
	CheckBox				(char*, int, int, int);
	CheckBox				(char*, int, int, int, int, int);

	void Init				(Display *, int, int);
	~CheckBox				()
		{if (Drawn) Erase();}
	void	Draw				();
	void Erase			();
	BOOLEAN ButtonBounds	(int, int);
	Message Dispatch		(Message &);
	int GetXDim			();
	int GetYDim			();
	void ChangeStatus		(BOOLEAN);
	int BoxId				();
	int CenterPtX			();
	int CenterPtY			();
	void SetPseudo			(BOOLEAN);
	void DisableBox			();
	void EnableBox			();
protected:
	void animate_box	();
	void animate_frame	();
	Message mouse_msg	(Message &);
	Message key_msg	(Message &);
	Message command_msg	(Message &);
	BOOLEAN status		();
	void disable_box	();
	void enable_box	();
private:
	void set_constants	(char*, int, int, int, int);
	void	image_size	();
	void	image_size	(int, int);
	void prep_screen	(int);
	void name_button	(int);
	void fill_box		();
	void empty_box		(int);
	void draw_check	();
};

inline CheckBox::CheckBox(char *name, int color, int fcolor, int Id, int type)
	{set_constants(name, color, fcolor, Id, type); image_size();
	xdim=X2-X; ydim=Y2-Y;}

inline CheckBox::CheckBox(char *name, int color, int fcolor, int Id, int type, int x2, int y2)
	{set_constants(name, color, fcolor, Id, type); image_size(x2, y2);
	xdim=X2-X; ydim=Y2-Y;}

inline CheckBox::CheckBox(char *name, int color, int Id, int type)
	{set_constants(name, color, BLUE, Id, type); image_size();
	xdim=X2-X; ydim=Y2-Y;}

inline CheckBox::CheckBox(char *name, int color, int Id, int type, int x2, int y2)
	{set_constants(name, color, BLUE, Id, type); image_size(x2, y2);
	xdim=X2-X; ydim=Y2-Y;}

inline void CheckBox::ChangeStatus(BOOLEAN bswitch)
	{BStatus=bswitch; animate_box();}

inline int CheckBox::BoxId()
	{return IdNum;}

inline int CheckBox::CenterPtX()
	{return X+BOXSIZE/2;}

inline int CheckBox::CenterPtY()
	{return Y+BOXSIZE/2;}

inline void CheckBox::SetPseudo(BOOLEAN state)
	{Able=state;}

inline void CheckBox::DisableBox()
	{disable_box();}

inline void CheckBox::EnableBox()
	{enable_box();}

inline void CheckBox::image_size(int x2, int y2)
	{X2 = X+x2; Y2 = Y+y2;}

inline void CheckBox::image_size()
	{X2 = X+string_width+(2*CBORDERX) + (BOXSIZE+BOXBORDER);
	Y2 = Y+((BOXSIZE>string_height)?BOXSIZE:string_height);}

inline BOOLEAN CheckBox::status()
	{return BStatus;}

inline void CheckBox::animate_box()
	{if (BStatus) fill_box(); else empty_box(BOXCOLOR);}

inline int CheckBox::GetXDim()
	{return xdim;}

inline int CheckBox::GetYDim()
	{return ydim;}

inline void CheckBox::disable_box()
	{Able=No; empty_box(OFFCOLOR); name_button(OFFCOLOR);}

inline void CheckBox::enable_box()
	{Able=Yes; empty_box(BOXCOLOR); name_button(Color);}



#endif
