/*************************************************************************
 *
 * 	FILE:	mapwin.c
 *	AUTHOR:	Wilson Tham
 *	DATE:	30 April 1993
 *	PURPOSE:
 *
 **************************************************************************/

#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include "mapwin.h"
#include "map.h"

const unsigned lines[3] = { 0xFC00, 0x03E0, 0x001F};


MapWindow::MapWindow (void)
{
	XDim = MapWinXDim;  YDim = MapWinYDim;
	CliX1 = CliY1 = CliX2 = CliY2 = 0;
	TX1 = TY1 = TX2 = TY2 = 0;
	BorderColor = RED;
	BkColor = LIGHTGRAY;
	TitleColor = WHITE;
	Title = NULL;
	Title = new char[80];
	strcpy(Title, "- Ye Olde Map of London -");
	ClientD = TitleD = NULL;
	NodeList = NULL;
	NNodes = 0;
	Drawn = False;
}


MapWindow::~MapWindow (void)
{
  delete NodeList;
}


void MapWindow::Init (Display *d, int x, int y)
{
	Disp = d;
	X = x;  Y = y;

	CalcBorder ();
	if (Title) {
		CalcTitle ();
		TitleD = new Display (Disp, TX1, TY1, TX2, TY2);
		assert (TitleD != 0);
		TitleD->SetBkColor (BorderColor);
		TitleD->SetTextJustify (CENTER_TEXT, TOP_TEXT);
		TitleD->SetTextStyle (TitleFont, HORIZ_DIR, TitleFontSize);
		TitleD->SetColor (TitleColor);
	}

	ClientD = new Display (Disp, CliX1, CliY1, CliX2, CliY2);
	assert (ClientD != 0);
	ClientD->SetBkColor (BkColor);
	AllocNodes ();
}


void MapWindow::AllocNodes (void)
{
	int px, py;

	NNodes = theMap.num_nodes ();
	NodeList = new Nodes* [NNodes+1];
		assert (NodeList != 0);

	for (int i=1; i<=NNodes; i++) {
		NodeList[i] = new Nodes(i, Off);
		assert (NodeList[i] != NULL);
		px=theMap.x(i); py=theMap.y(i);
		Scale (px, py);
		ClientD->Add(NodeList[i], px, py);
	}
}


void MapWindow::Draw (void)
{
	if (!Drawn) {
		assert (Disp != NULL);
		Disp->FastDraw ();
		PrepScreen ();
		DrawBorder ();

		if (TitleD)
			DrawTitle ();

		ClientD->Clear ();
		DrawLines ();
		ClientD->Draw ();
		Disp->Done ();
		Drawn = True;
	}
}

void MapWindow::EnableNodes(MoveList &mlist, int color)
{
	if(mlist.Reset()) {
		do {
			NodeList[mlist.GetCurrent()->Node]->EnableNode(color);
		} while(mlist.Next());
	}
}

void MapWindow::DisableNodes(MoveList &mlist)
{
	if(mlist.Reset()) {
		do {
			NodeList[mlist.GetCurrent()->Node]->DisableNode();
		} while(mlist.Next());
	}
}

void MapWindow::HilightNode(int idnum, int color)
{
	NodeList[idnum]->ChangeStatus(On, color);
}

void MapWindow::UnHilightNode(int idnum)
{
	NodeList[idnum]->ChangeStatus(Off);
}

void MapWindow::DrawLines (void)
{
	int num_lines=0, node, i, j, k;

	for(i=1; i<=theMap.num_nodes(); i++) {
		node= i;
		num_lines=theMap.num_connections(i);
		for(j=0; j<num_lines; j++)
		{
		  int ToNode = theMap.connected_to (i, j);
		  for (k= 0; k < theMap.num_connections(i, j); k++) {
			ConnectNode(node, theMap.connected_to(i, j, k));
			node= theMap.connected_to(i, j, k);
		  }
		  if (i < ToNode)
			ConnectNode(node, theMap.connected_to(i, j));
		}
	}
}

void MapWindow::ConnectNode(int node1, int node2)
{
	const int Spacing=2;

	int x, y, x2, y2, i=0, px1[3], py1[3], px2[3], py2[3], face;

	x=theMap.x(node1); y=theMap.y(node1);
	x2=theMap.x(node2); y2=theMap.y(node2);

	Scale (x, y);
	Scale (x2, y2);

	i=0;
	if (theMap.taxi(node1, node2)) i++;
	if (theMap.bus(node1, node2)) i++;
	if (theMap.subway(node1, node2)) i++;

	if((x<x2)&&(y>y2)) face=0;
	else if((x>x2)&&(y>y2)) face=1;
	else if((x>x2)&&(y<y2)) face=0;
	else if((x<x2)&&(y<y2)) face=1;
	else face=1;

	switch(face) {
		case 0:
			px1[0]=x-Spacing;	py1[0]=y-Spacing;
			px1[1]=x; 		py1[1]=y;
			px1[2]=x+Spacing;	py1[2]=y+Spacing;
			px2[0]=x2-Spacing;	py2[0]=y2-Spacing;
			px2[1]=x2; 		py2[1]=y2;
			px2[2]=x2+Spacing;	py2[2]=y2+Spacing;
			break;
		case 1:
			px1[0]=x+Spacing;	py1[0]=y-Spacing;
			px1[1]=x; 		py1[1]=y;
			px1[2]=x-Spacing;	py1[2]=y+Spacing;
			px2[0]=x2+Spacing;	py2[0]=y2-Spacing;
			px2[1]=x2; 		py2[1]=y2;
			px2[2]=x2-Spacing;	py2[2]=y2+Spacing;
			break;
	}

	int which;
	for (int count=0; count<i; count++) {
		ClientD->SetLineStyle(SOLID_LINE, 0, Spacing);
		switch(count) {
			case 0:	// Taxi
				if(theMap.taxi(node1, node2))
					ClientD->SetColor(YELLOW), which=0;
				else if (theMap.bus (node1, node2))
					ClientD->SetColor(GREEN), which=1;
				else {
					ClientD->SetColor (RED);
					ClientD->SetLineStyle(SOLID_LINE, 0, 3);
					which=2;
				}
				break;
			case 1:
				if(theMap.subway(node1, node2)) {
					ClientD->SetColor (RED);
					ClientD->SetLineStyle(SOLID_LINE, 0, 3);
					which=2;
				}
				else if (theMap.bus (node1, node2))
					ClientD->SetColor(GREEN), which=1;
				else
					ClientD->SetColor(YELLOW), which=0;
				break;
			case 2:
				if(theMap.bus(node1, node2))
					ClientD->SetColor(GREEN), which=1;
				else if (theMap.taxi (node1, node2))
					ClientD->SetColor(YELLOW), which=0;
				else {
					ClientD->SetColor (RED);
					ClientD->SetLineStyle(SOLID_LINE, 0, 3);
					which=2;
				}
				break;
		}

		ClientD->Line(px1[which], py1[which], px2[which], py2[which]);
	}
	ClientD->SetLineStyle(SOLID_LINE, 0, 1);
}

/*
void MapWindow::ConnectNode(int node1, int node2)
{
	int x, y, x2, y2, i=0;

	x=theMap.x(node1); y=theMap.y(node1);
	x2=theMap.x(node2); y2=theMap.y(node2);

	Scale (x, y);
	Scale (x2, y2);

//	i=theMap.taxi(node1, node2)+theMap.bus(node1, node2)+
//	  theMap.subway(node1, node2);

	for(int count=0; count<3; count++) {
		ClientD->SetLineStyle(USERBIT_LINE, lines[count], TransWidth);
		switch(count) {
			case 0:	// Taxi
				if(theMap.taxi(node1, node2))
					ClientD->SetColor(YELLOW);
				else if (theMap.bus (node1, node2))
					ClientD->SetColor(GREEN);
				else
					ClientD->SetColor (RED);
				break;
			case 1:
				if(theMap.bus(node1, node2))
					ClientD->SetColor(GREEN);
				else if (theMap.taxi (node1, node2))
					ClientD->SetColor(YELLOW);
				else
					ClientD->SetColor (RED);
				break;
			case 2:
				if(theMap.subway(node1, node2))
					ClientD->SetColor(RED);
				else if (theMap.taxi (node1, node2))
					ClientD->SetColor(YELLOW);
				else
					ClientD->SetColor(GREEN);
				break;
		}
		ClientD->Line(x, y, x2, y2);
	}
	ClientD->SetLineStyle(SOLID_LINE, 0, 1);
}
*/
void MapWindow::Scale (int& x, int& y)
{
	const float ScreenPortion=.98;

	int maxx, maxy;
	maxx = theMap.max_x ();
	maxy = theMap.max_y ();

	float mx=ClientD->GetMaxX()*ScreenPortion;
	float my=ClientD->GetMaxY()*ScreenPortion;

	x = x * (mx / maxx);
	y = y * (my / maxy);
}


