/**********************************************************************
*	FILE:	MESSAGES.H
*	AUTHOR:	CALeague
*	DATE:	18 April 1993
*	PURPOSE:
*	  Messages are used to represent mouse and keyboard events, as
*	  well as to inform control functions of event that occur at
*	  the GUI level, for example, the pressing of a button.
***********************************************************************/

#ifndef _MESSAGES_H
#define _MESSAGES_H

#include "typedefs.h"


/**********************************************************************
*	Message Structure
***********************************************************************/

struct Message {
  unsigned origin;
  unsigned code;	// Event code or message type
  int d1, d2;		// Integer data
  char str [25];	// String data

  Message (void) { origin = code = d1 = d2 = 0; str [0] = 0; }
  Message (unsigned o, unsigned c)
    { origin = o; code = c; d1 = d2 = 0; str [0] = 0; }
  Message (unsigned o, unsigned c, int a)
    { origin = o; code = c; d1 = a; d2 = 0; str [0] = 0; }
  Message (unsigned o, unsigned c, int a, int b)
    { origin = o; code = c; d1 = a; d2 = b; str [0] = 0; }
  bool operator == (Message &);
  bool operator != (Message &msg) { return (bool) (! (*this == msg)); }
};


/**********************************************************************
*	Predefined Message Origins
***********************************************************************/

const unsigned GodMsg    = 0x0000;
const unsigned Unknown	 = 0xFFFF;
const unsigned MouseMsg	 = 0x0002;
const unsigned KeyMsg	 = 0x0001;
const unsigned ButtonMsg = 0x0010;

/**********************************************************************
*	Predefined Message Types
***********************************************************************/

const unsigned Null		= 0x0000;	// Normal stuff
const unsigned Idle		= 0x0000;
const unsigned String		= 0x0001;
const unsigned Acknowledgement	= 0x0002;

const unsigned MouseDown	= 0x0020;	// Mouse stuff
const unsigned LMouseDown	= 0x0020;
const unsigned RMouseDown	= 0x0021;
const unsigned MouseStillDown	= 0x0022;
const unsigned LMouseStillDown	= 0x0022;
const unsigned RMouseStillDown	= 0x0023;
const unsigned MouseUp		= 0x0024;
const unsigned LMouseUp		= 0x0024;
const unsigned RMouseUp		= 0x0025;

const unsigned KeyPressed 	= 0x0010;      // Keyboard

const unsigned ButtonPress   	= 0x0101;      // Buttons
const unsigned RequestStatus 	= 0x0102;
const unsigned Status        	= 0x0103;
const unsigned Enable		= 0x0104;
const unsigned Disable		= 0x0105;


/**********************************************************************
*	Predefined Message
***********************************************************************/

extern Message NullMsg;
extern Message Acknowledged;


#endif
