/*************************************************************************
* 	FILE:	msmouse.h
*	AUTHOR: Weiskamp, Heiny, Flamig
*	DATE:	18 April 1993
*	PURPOSE:
*	  Header for MSMouse class to interface with Microsoft mouse
*	  driver
**************************************************************************
*			REVISION HISTORY
*
*  Rev	Date		Analyst
*
*    0	??		Weiskamp, Heiny, Flamig
*			  Designed.
*    1	16 Apr 1993  	Wilson Tham, Steven Winig
*			  Added 640x480 support, a few additional features.
*    2	18 Apr 1993	CALeague
*			  Added support for my message-passing protocols.
**************************************************************************/

#ifndef _MSMOUSE_H
#define _MSMOUSE_H

#include "typedefs.h"
#include "messages.h"

const int THRESHOLD = 16;

// Defines graphics mouse cursor styles

struct HotSpotStruct {int X, Y;};

struct MouseCursor {
	HotSpotStruct HotSpot;
	unsigned ScreenMask[16];
	unsigned CursorMask[16];
};

extern const MouseCursor ArrowCursor;
extern const MouseCursor HandCursor;
extern const MouseCursor LeftRightCursor;
extern const MouseCursor UpDownCursor;


// Mouse Button Masks
const unsigned LeftButton		= 0x0001;
const unsigned RightButton		= 0x0002;

// The video modes that the mouse is running under
enum VideoModeType {TextScrn, LowResGr, HighResGr, Graphics};

// The mouse object class
class MSMouse {
protected:
	int  OldX, OldY;	// Used solely by Moved to keep position
	bool OK;		// True if mouse initialized
	bool MouseOff;		// True is mouse is disabled (Default)
	bool TextMode;		// True if in text mode
	bool LowRes;
	bool HighRes;		// True is in 640 x 480 graphics mode

	bool DriverExists (void);
	int  PressCnt (unsigned ButtonMask);
	int  ReleaseCnt	(unsigned ButtonMask);
	bool Moved (void);
	bool Operating (void) { return (bool) (!MouseOff); }
	unsigned Status (int &Mx, int &My);
	unsigned ButtonStatus (void);
	int X, Y, Dx, Dy;	// Keeps track of the mouse's movement
public:
	MSMouse	(void);
	MSMouse (VideoModeType VideoMode);
	void Setup  (VideoModeType VideoMode);
	bool SetupOK (void) { return OK; }
	void Hide (void);
	void Show (void);
	void Move (int Mx, int My);
	void TurnOn (void);
	void TurnOff (void);
	void SetGCursor	(const MouseCursor &NewCursor);
	void SetMouseCursor (int, int);
	void SetSpeed2 ();

	Message Event (void);
	Message WaitForAnyEvent (void);
	Message WaitForEvent (Message&);
	void Flush (void);
};

extern MSMouse Mouse;		// Global Instance

#endif

