/*************************************************************************
* 	FILE:	scrnobj.c
*	AUTHOR: CALeague
*	DATE:	25 April 1993
*	PURPOSE:
*	  Contains functions which handle a list of ScreenObjects.
*	  A ScreenObject is any interactive object which can be drawn on
*	  the screen, like a window or a button.
**************************************************************************/

#include "scrnobj.h"


void ScrnObjList::Insert (ScrnObj *a)
{
  a->Next = First;
  First = a;
  if (!Current)
    Current = First;
  if (!Last)
    Last = First;
}


void ScrnObjList::Append (ScrnObj *a)
{
  if (Last)
  {
    Last->Next = a;
    Last = a;
  }
  else
    Insert (a);
}


void ScrnObjList::Remove (ScrnObj *a)
{
  if (First == a)
  {
    ScrnObj* tmp = First;
    First = First->Next;
    delete tmp;
  }
  else
  {
    ScrnObj* prev = First;
    ScrnObj* curr = First->Next;
    while (curr)
    {
      if (curr == a)
      {
	prev->Next = curr->Next;
	if (curr == Last)
	  Last = prev;
	delete curr;
	curr = NULL;
      }
      else
      {
	prev = curr;
	curr = curr->Next;
      }
    }
  }
  Current = First;
}


bool ScrnObjList::Reset (void)
{
  Current = First;
  return (bool) (Current);
}


bool ScrnObjList::Next (void)
{
  if (Current)
    Current = Current->Next;
  return (bool) (Current);
}


void ScrnObjList::Nuke (void)
{
  delete First;
  First = Current = Last = NULL;
}



