/*************************************************************************
* 	FILE:	scrnobj.h
*	AUTHOR: CALeague
*	DATE:	25 April 1993
*	PURPOSE:
*	  A ScreenObject is any interactive object which can be drawn on
*	  the screen, like a window or a button.  It generates and
*	  responds to messages.  ScrnObj is a virtual base class;
*	  it exists only so that various types of windows and buttons
*	  can inherit from it.
**************************************************************************/

#ifndef _SCRNOBJ_H
#define _SCRNOBJ_H

#include <stddef.h>
#include "messages.h"

class Display;				// foreward declaration


class ScrnObj {
  friend class ScrnObjList;
protected:
  ScrnObj* Next;
  Display *Disp;      // Display upon which the object exists
  int X, Y;           // Top-Left of object (relative to Disp)
  bool Drawn;
public:
  ScrnObj ()  { Disp = NULL; Next = NULL; }
  virtual ~ScrnObj () { delete Next; }
  virtual int GetXDim (void) { return 0; }
  virtual int GetYDim (void) { return 0; }
  virtual void Init (Display *d, int x, int y)
    { Disp = d;  X = x;  Y = y; }
  virtual void Draw () = 0;
  virtual void Erase () {}
  virtual Message Dispatch (Message&)  { return NullMsg; }
};


class ScrnObjList {
  ScrnObj *First;
  ScrnObj *Last;
  ScrnObj *Current;
public:
  ScrnObjList () { First = Last = Current = NULL; }
  ~ScrnObjList () { Nuke (); }
  void Insert (ScrnObj *a);
  void Append (ScrnObj *a);
  void Remove (ScrnObj *a);
  ScrnObj* GetCurrent (void) { return Current; }
  bool Reset (void);
  bool Next (void);
  void Nuke (void);
};



#endif
