/**********************************************************************
*	FILE:	ui.h
*	AUTHOR:	CALeague
*	DATE:	30 April 1993
*	PURPOSE:
*	  Houses the user interface
***********************************************************************/

#ifndef _UI_H
#define _UI_H

#include "players.h"
#include "display.h"
#include "mapwin.h"
#include "keywin.h"
#include "cpanel.h"
#include "playrwin.h"
#include "infowin.h"


// Player colors
const int PColor[6] ={LIGHTCYAN, WHITE, LIGHTGREEN, LIGHTBLUE,
		      YELLOW, MAGENTA};


const int ID_QUIT    = 0xFF;
const int ID_SPY     = 0xFE;
const int ID_REFRESH = 0xFD;
const int ID_LEGAL   = 0xFC;

class UserInterface
{
  MapWindow    *mapwin;
  KeyWindow    *keywin;
  CPanelWindow *cpanel;
  PlayerWindow *playwin;
  InfoWindow   *infowin;

  void ShowSpyMoves (MoveList& moves);
  int ComputePossTokens (MoveList&, int dest);

public:
  void FlashNodes (Player& p, MoveList& ml);

  void ShowAllPlayers (void);
  Move GetPlayerMove (Player&, MoveList&);
  void BeginTurn (Player&);
  void EndTurn (Player&, Move);
  void RevealSpy (Player& p, Move m);
  void SpyTurn (Move&, int);
  Move NoLegalMoves (Player& p);
  int TransPop(byte bv, int color);


  void InitScreen (void);
  void DrawScreen (void) { theDisplay.Draw (); }
  void EraseScreen (void) { theDisplay.Erase (); theDisplay.Clear (); }
  void Refresh (void) { EraseScreen(); DrawScreen (); }

  void DrawTitleScreen();
  int* UFWOOM (void);
  void Intro();
  void AboutButton(Display&);
  void AnnounceWin (Player&, int);
};

extern UserInterface UI;	// Global instance


#endif
