/**********************************************************************
*	FILE:	viewport.c
*	AUTHOR:	CALeague
*	DATE:	25 April 1993
*	PURPOSE:
*	  Contains functions pertaining to Viewports.
***********************************************************************/

#include "viewport.h"

/**********************************************************************
*	Static data members
***********************************************************************/

Viewport* Viewport::Context = NULL;
int Viewport::Fast = 0;


/**********************************************************************
*	Constructors
***********************************************************************/

Viewport::Viewport (Viewport* parent, int x1, int y1, int x2, int y2)
: SView (x1, y1, x2, y2)
{
  Parent = parent;
  if (Parent)
  {
    SView.X1 += Parent->SView.X1;
    SView.Y1 += Parent->SView.Y1;
    SView.X2 += Parent->SView.X1;
    SView.Y2 += Parent->SView.Y1;
  }
  Install ();
}

Viewport::Viewport (void)
{
  Parent = NULL;
  Install ();
}


void Viewport::Install (void)
{
  SView.Install ();
  SColor.Install ();
  SFill.Install ();
  SLine.Install ();
  SText.Install ();
  SPos.Install ();
  Context = this;
}


void Viewport::Clear (void)
{
  SwitchTo ();
  DoDraw ();
  setfillstyle (SOLID_FILL, SColor.BG);
  bar (0, 0, SView.X2, SView.Y2);
  DoneDraw ();
  SPos.X = SPos.Y = 0;
  Context = NULL;
}


int Viewport::TransX (int x)
{
  if (!Parent)
    return x;
  else
    return (x - (Parent->TransX (SView.X1)));
}

int Viewport::TransY (int y)
{
  if (!Parent)
    return y;
  else
    return (y - (Parent->TransY (SView.Y1)));
}


bool Viewport::InBounds (int x, int y)
{
  return (bool) ((x >= 0) && (x <= GetMaxX()) &&
		 (y >= 0) && (y <= GetMaxY()));
}

